
/******************************************************************************/
/* ProSQL - SQL integration for Progress systems                              */
/* Copyright (c) 20005, Scott Auge                                            */
/* All rights reserved.                                                       */
/*                                                                            */
/* Redistribution and use in source and binary forms, with or without         */
/* modification, are permitted provided that the following conditions         */
/* are met:                                                                   */
/*                                                                            */
/* * Redistributions of source code must retain the above copyright           */
/*   notice, this list of conditions and the following disclaimer.            */
/* * Redistributions in binary form must reproduce the above copyright        */
/*   notice, this list of conditions and the following disclaimer in          */
/*   the documentation and/or other materials provided with the distribution. */
/* * Neither the name of the Amduus Information Works, Inc. nor the names     */
/*   of its contributors may be used to endorse or promote products derived   */
/*   from this software without specific prior written permission.            */
/*                                                                            */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS        */
/* "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED  */
/* TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR */
/* PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR           */
/* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,      */
/* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,        */
/* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR         */
/* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF     */
/* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING       */
/* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS         */
/* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.               */
/******************************************************************************/

/* $headers$ 


DELETE FROM mytable WHERE where_clause

*/

{logentry.i}

DEFINE INPUT PARAMETER cSQL AS CHARACTER NO-UNDO.

DEFINE VARIABLE cTableName AS CHARACTER NO-UNDO.
DEFINE VARIABLE cWhere     AS CHARACTER NO-UNDO.
DEFINE VARIABLE cQuery     AS CHARACTER NO-UNDO.

DEFINE VARIABLE hBuffer    AS HANDLE NO-UNDO.
DEFINE VARIABLE hQuery     AS HANDLE NO-UNDO. /* Version 10 use hBuffer:FIND-FIRST */

DEFINE VARIABLE i          AS INTEGER NO-UNDO.

/************************************************************************/
/* Determine the table name                                             */
/************************************************************************/

ASSIGN cTableName = TRIM(ENTRY(3, cSQL, " ")).
RUN ipLog (2, "DELETING").
RUN ipLog (3, "Table Name = " + cTableName).

/************************************************************************/
/* Get our WHERE Clause                                                 */
/************************************************************************/

ASSIGN cWhere = TRIM(REPLACE (cSQL, "DELETE FROM " + cTableName, "")).
RUN ipLog (3, "Where = " + cWhere).

/************************************************************************/
/* Start the deleting process.  Remember deletes in SQL are set         */
/* oriented so we are going to look and delete until no more records    */
/* matching the where clause exist.                                     */
/************************************************************************/

CREATE BUFFER hBuffer FOR TABLE cTableName.

CREATE QUERY hQuery.
hQuery:ADD-BUFFER(hBuffer).

ASSIGN cQuery = "FOR EACH " + cTableName + " EXCLUSIVE-LOCK " + cWhere.
RUN ipLog (2, "Query is: " + cQuery).

IF hQuery:QUERY-PREPARE(cQuery) = FALSE THEN DO:
    RUN ipLog(2, "Query Failed Prepare!").
END.

IF hQuery:QUERY-OPEN() = FALSE THEN DO:
  RUN ipLog(2, "Open FAILED!").
  /* Begin Used for debugging */
  REPEAT i = 1 TO hQuery:NUM-BUFFERS:
    hBuffer = hQuery:GET-BUFFER-HANDLE(i).
    RUN ipLog (2, "Query Has Buffer: " + hBuffer:NAME).
  END.

END. /* IF lStatusFlag = FALSE */

REPEAT TRANSACTION:

  hQuery:GET-NEXT.
  IF (hQuery:QUERY-OFF-END) THEN LEAVE.

  hBuffer = hQuery:GET-BUFFER-HANDLE(1).

  RUN ipLog(2, "Deleting RowID: " + STRING(hBuffer:ROWID)).
  hBuffer:BUFFER-DELETE().

END. /* REPEAT */

