
/******************************************************************************/
/* ProSQL - SQL integration for Progress systems                              */
/* Copyright (c) 20005, Scott Auge                                            */
/* All rights reserved.                                                       */
/*                                                                            */
/* Redistribution and use in source and binary forms, with or without         */
/* modification, are permitted provided that the following conditions         */
/* are met:                                                                   */
/*                                                                            */
/* * Redistributions of source code must retain the above copyright           */
/*   notice, this list of conditions and the following disclaimer.            */
/* * Redistributions in binary form must reproduce the above copyright        */
/*   notice, this list of conditions and the following disclaimer in          */
/*   the documentation and/or other materials provided with the distribution. */
/* * Neither the name of the Amduus Information Works, Inc. nor the names     */
/*   of its contributors may be used to endorse or promote products derived   */
/*   from this software without specific prior written permission.            */
/*                                                                            */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS        */
/* "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED  */
/* TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR */
/* PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR           */
/* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,      */
/* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,        */
/* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR         */
/* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF     */
/* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING       */
/* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS         */
/* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.               */
/******************************************************************************/

/* $headers$ 


Oh how the 4GL needs a YACC/LEX.
INSERT INTO table (fieldlist) VALUES (values)

Escape slash your quotes in a quote

INSERT INTO mytable (myfield) VALUES ('Tom\'s Value')

*/

{logentry.i}

DEFINE INPUT PARAMETER cSQL AS CHARACTER NO-UNDO.

DEFINE VARIABLE cTableName AS CHARACTER NO-UNDO.
DEFINE VARIABLE cFieldList AS CHARACTER NO-UNDO.
DEFINE VARIABLE cValueList AS CHARACTER NO-UNDO.
DEFINE VARIABLE cCurField  AS CHARACTER NO-UNDO.
DEFINE VARIABLE cCurValue  AS CHARACTER NO-UNDO.
DEFINE VARIABLE cTemp1     AS CHARACTER NO-UNDO.

DEFINE VARIABLE iIter AS INTEGER NO-UNDO.
DEFINE VARIABLE iMaxIter AS INTEGER NO-UNDO.

DEFINE VARIABLE hBuffer AS HANDLE NO-UNDO.
DEFINE VARIABLE hField  AS HANDLE NO-UNDO.

/**********************************************************************/
/* Since we can't straight parse based on a , since it might be in a  */
/* string, we need to walk the string from start to back!             */
/**********************************************************************/

FUNCTION ValueListEntry RETURNS CHARACTER (INPUT iEntry AS INTEGER, INPUT cList AS CHARACTER):

  DEFINE VARIABLE lInString AS LOGICAL NO-UNDO.
  DEFINE VARIABLE cCurChar  AS CHARACTER NO-UNDO.
  DEFINE VARIABLE cLastChar AS CHARACTER NO-UNDO.
  DEFINE VARIABLE iCurEntry AS INTEGER NO-UNDO.
  DEFINE VARIABLE iCurPos   AS INTEGER NO-UNDO.
  DEFINE VARIABLE iNumPos   AS INTEGER NO-UNDO.
  DEFINE VARIABLE cEntryValue AS CHARACTER NO-UNDO.

  ASSIGN iNumPos = LENGTH(cList)
         iCurEntry = 1
         lInString = NO
         cCurChar = "".

  DO iCurPos = 1 TO iNumPos:

    ASSIGN cLastChar = cCurChar.
    ASSIGN cCurChar = SUBSTRING(cList, iCurPos, 1).
    IF cCurChar = "'" AND cLastChar <> "\\" THEN DO:
      ASSIGN lInString = NOT lInString.
      NEXT.
    END.

    IF NOT lInString AND cCurChar = "," THEN DO:
      ASSIGN iCurEntry = iCurEntry + 1.
      NEXT.
    END.

    IF iCurEntry > iEntry THEN LEAVE.
    IF iCurEntry <> iEntry THEN NEXT.

    IF cCurChar = "\\" AND cLastChar <> "\\" THEN NEXT.
    ASSIGN cEntryValue = cEntryValue + cCurChar.

  END. /* DO iCurPos = 1 TO iNumPos */

  RETURN cEntryValue.

END. /* PROCEDURE ValueListEntry */

/**********************************************************************/
/* Clean out the extra spaces and the like so parsing above works     */
/**********************************************************************/

FUNCTION CleanValueList RETURNS CHARACTER (INPUT cList AS CHARACTER):

  DEFINE VARIABLE lInString AS LOGICAL NO-UNDO.
  DEFINE VARIABLE cCurChar  AS CHARACTER NO-UNDO.
  DEFINE VARIABLE cLastChar AS CHARACTER NO-UNDO.
  DEFINE VARIABLE iCurPos   AS INTEGER NO-UNDO.
  DEFINE VARIABLE iNumPos   AS INTEGER NO-UNDO.
  DEFINE VARIABLE cEntryValue AS CHARACTER NO-UNDO.

  ASSIGN iNumPos = LENGTH(cList)
         lInString = NO
         cCurChar = "".

  DO iCurPos = 1 TO iNumPos:

    ASSIGN cLastChar = cCurChar.
    ASSIGN cCurChar = SUBSTRING(cList, iCurPos, 1).
    IF cCurChar = "'" AND cLastChar <> "\\" THEN
      ASSIGN lInString = NOT lInString.

    IF NOT lInString AND cCurChar = " " THEN NEXT.

    ASSIGN cEntryValue = cEntryValue + cCurChar.

  END. /* DO iCurPos = 1 TO iNumPos */

  RETURN cEntryValue.

END. /* FUNCTION CleanValueList() */

/**********************************************************************/
/* In case we get here by mistake.                                    */
/**********************************************************************/

IF NOT cSQL BEGINS "INSERT" THEN RETURN.

/**********************************************************************/
/* Find out the name of the table we are inserting into.              */
/**********************************************************************/

ASSIGN cTableName = ENTRY(3, cSQL, " ").

RUN ipLog (2, "INSERTING").
RUN ipLog (2, "Table Name is: " + cTableName).


/**********************************************************************/
/* Remove the first three tokens INSERT INTO cTable                   */
/**********************************************************************/

ASSIGN cSQL = REPLACE(cSQL, "INSERT INTO " + cTableName, "").

RUN ipLog (3, "Parsing: " + cSQL).

ASSIGN cSQL = REPLACE(cSQL, " VALUES ", "~n")
       cFieldList = ENTRY(1, cSQL, "~n")
       cFieldList = REPLACE (cFieldList, "(", "")
       cFieldList = TRIM(REPLACE (cFieldList, ")", ""))
       cValueList = TRIM(ENTRY(2, cSQL, "~n"))
       cValueList = SUBSTRING(cValueList, 2, LENGTH(cValueList) - 2)
       cValueList = CleanValueList(cValueList)
       .

RUN ipLog (3, "Field List: " + cFieldList).
RUN ipLog (3, "Value List: " + cValueList).

/**********************************************************************/
/* Create a dynamic buffer and toss the goods into it.                */
/**********************************************************************/

DO TRANSACTION:

CREATE BUFFER hBuffer FOR TABLE cTableName.
hBuffer:BUFFER-CREATE().

ASSIGN iMaxIter = NUM-ENTRIES(cFieldList).

DO iIter = 1 TO iMaxIter:

  ASSIGN hField = hBuffer:BUFFER-FIELD(TRIM(ENTRY(iIter, cFieldList))).

  CASE hField:DATA-TYPE:

    WHEN "CHARACTER" THEN DO:
      ASSIGN cTemp1 = ValueListEntry(iIter, cValueList).
      RUN ipLog (2, "Buffer Field Name = " + hField:Name).
      RUN ipLog (2, "Character Value = " + cTemp1).
      ASSIGN hField:BUFFER-VALUE = STRING(ValueListEntry(iIter, cValueList)).

    END.

    WHEN "INTEGER" THEN DO:
      ASSIGN cTemp1 = ValueListEntry(iIter, cValueList).
      RUN ipLog (2, "Buffer Field Name = " + hField:Name).
      RUN ipLog (2, "Integer Value = " + cTemp1).
      ASSIGN hField:BUFFER-VALUE = INTEGER(ValueListEntry(iIter, cValueList)).

    END.

    WHEN "DECIMAL" THEN DO:
      ASSIGN cTemp1 = ValueListEntry(iIter, cValueList).
      RUN ipLog (2, "Buffer Field Name = " + hField:Name).
      RUN ipLog (2, "Decimal Value = " + cTemp1).
      ASSIGN hField:BUFFER-VALUE = DECIMAL(ValueListEntry(iIter, cValueList)).

    END.

    WHEN "LOGICAL" THEN DO:
      ASSIGN cTemp1 = ValueListEntry(iIter, cValueList).
      RUN ipLog (2, "Buffer Field Name = " + hField:Name).
      RUN ipLog (2, "Logical Value = " + cTemp1).
      ASSIGN hField:BUFFER-VALUE = IF cTemp1 = "true" THEN TRUE
                                   ELSE IF cTemp1 = "false" THEN FALSE
                                   ELSE ?.

    END.

  END. /* CASE */

END. /* DO iIter = 1 TO iNumFields */

END.
/* We're done, commit automatically */

