
/******************************************************************************/
/* ProSQL - SQL integration for Progress systems                              */
/* Copyright (c) 20005, Scott Auge                                            */
/* All rights reserved.                                                       */
/*                                                                            */
/* Redistribution and use in source and binary forms, with or without         */
/* modification, are permitted provided that the following conditions         */
/* are met:                                                                   */
/*                                                                            */
/* * Redistributions of source code must retain the above copyright           */
/*   notice, this list of conditions and the following disclaimer.            */
/* * Redistributions in binary form must reproduce the above copyright        */
/*   notice, this list of conditions and the following disclaimer in          */
/*   the documentation and/or other materials provided with the distribution. */
/* * Neither the name of the Amduus Information Works, Inc. nor the names     */
/*   of its contributors may be used to endorse or promote products derived   */
/*   from this software without specific prior written permission.            */
/*                                                                            */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS        */
/* "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED  */
/* TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR */
/* PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR           */
/* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,      */
/* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,        */
/* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR         */
/* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF     */
/* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING       */
/* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS         */
/* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.               */
/******************************************************************************/

/*
 * ProSQL Build 2005216111147
 * Poor man's SQL Server.  You can connect to this process and issue
 * SQL statements to it from a socket connection. 
 * This is for that stuff that the normal drivers just won't link with
 * or are on unsupported Progress OS's (aka Apple's OS X).
 *
 */


DEFINE VARIABLE hServer AS HANDLE NO-UNDO.

&GLOBAL-DEFINE CR CHR(13) + CHR(10)
&GLOBAL-DEFINE VERSION 2005216111147

/* Placement of this include is important - uses the above vars */

{logentry.i NEW}

/*******************************************************************/
/* Borrowed this from Solution 20082 on the Progress Knowledgebase */
/*******************************************************************/

FUNCTION socketRead RETURNS CHARACTER (h AS HANDLE, l AS INTEGER):

  DEFINE VARIABLE b AS MEMPTR NO-UNDO.
  DEFINE VARIABLE s AS CHARACTER NO-UNDO.

  SET-SIZE(b) = l + 1.
  h:READ(b, 1, l, 1).
  s = GET-STRING(b,1).
  SET-SIZE(b) = 0.
  RETURN s.

END. /* FUNCTION socketRead () */

/*******************************************************************/
/* Begin executable code.                                          */
/* Get our command line parameters                                 */
/*******************************************************************/

ASSIGN scSocketPort = OS-GETENV("PORT")
       scLogFile = OS-GETENV("LOGFILE")
       scLogLevel = OS-GETENV("LOGLEVEL").

ASSIGN siLogLevel = INTEGER(scLogLevel) NO-ERROR.
IF siLogLevel = ? THEN siLogLevel = 0.

/******************************************************************/
/* Start logging if any.                                          */
/******************************************************************/

IF scLogFile <> "" THEN DO:
  OUTPUT TO VALUE(scLogFile) UNBUFFERED.
  RUN ipLog (1, "Starting!").
  RUN ipLog (1, "Build: {&VERSION}").
  RUN ipLog (1, "Port: " + scSocketPort).
  RUN ipLog (1, "LogFile: " + scLogFile).
  RUN ipLog (1, "LogLevel: " + STRING(siLogLevel)).
END.

/* Get our server socket prepped */

CREATE SERVER-SOCKET hServer NO-ERROR.

/******************************************************************/
/* When progress gets a connection, it will throw an event and    */
/* we capture it at PROCEDURE Connection.                         */
/******************************************************************/

hServer:SET-CONNECT-PROCEDURE("Connection").
hServer:ENABLE-CONNECTIONS("-S " + STRING(scSocketPort)).

WAIT-FOR WINDOW-CLOSE OF CURRENT-WINDOW.

/******************************************************************/
/* Server sockets will hand off a connection and wait for another */
/* one.  Here we start working with the connection we were given. */
/******************************************************************/

PROCEDURE Connection:

  DEFINE INPUT PARAMETER hSocket AS HANDLE.  /*Socket implicitly created*/

  RUN socketWrite (hSocket, "ProSQL Version {&VERSION} Waiting" + {&CR}).
  RUN ipLog (1,"Caught a connection from " + hSocket:REMOTE-HOST + "!").

  hSocket:SET-READ-RESPONSE-PROCEDURE ("ConnectionHandler", THIS-PROCEDURE).

END. /* PROCEDURE Connection */

/******************************************************************/
/*  Write data out to the socket!                                 */
/******************************************************************/
PROCEDURE socketWrite:

  DEFINE INPUT PARAMETER hSocket AS HANDLE NO-UNDO.
  DEFINE INPUT PARAMETER cText AS CHARACTER NO-UNDO.

  DEFINE VARIABLE mData AS MEMPTR NO-UNDO.

  SET-SIZE(mDATA) = LENGTH(cText) + 1.
  PUT-STRING(mData, 1) = cText.
  hSocket:WRITE(mData, 1, LENGTH(cText)).
  SET-SIZE (mData) = 0.

END. /* PROCEDURE socketWrite */

/******************************************************************/
/* Here is where we get most of the work done.  Pull the SQL off  */
/* the socket and start figuring out what to do with it.          */
/* Borrowed some of this from the above mentioned knowledgebase.  */
/******************************************************************/

PROCEDURE ConnectionHandler:

  DEFINE VARIABLE l AS INTEGER NO-UNDO.
  DEFINE VARIABLE cSQL AS CHARACTER NO-UNDO.

  RUN ipLog (4, "Handling input!").

  l = SELF:GET-BYTES-AVAILABLE().

  /* Sometimes this happens yep.  Usually telnets to check */
  /* for listening.                                        */

  IF l = 0 THEN DO:

    RUN ipLog (1, "Whoa, no data, closing").
    SELF:DISCONNECT().
    RETURN.

  END. /* IF l = 0 */

  /* Do to socket "noise" one uses <cmd>the statement</cmd> to isolate */
  /* the goods within good data from the socket.  This noise usually   */
  /* seems to be at the end of the message.                            */
  /* In the future this lets us add additional goods to the request.   */

  ASSIGN cSQL = socketRead(SELF:HANDLE, l).
  RUN ipLog (4, "Received: " + cSQL).

  ASSIGN cSQL = SUBSTRING(cSQL, 6)
         cSQL = SUBSTRING(cSQL, 1, INDEX(cSQL, "</cmd>") - 1).
   
  RUN ipLog (3, "SQL is: " + cSQL).
  RUN ParseStatement (cSQL, SELF:HANDLE).

  IF SELF:CONNECTED() THEN RUN socketWrite (SELF, "ProSQL Version {&VERSION} Waiting" + {&CR}).

END. /* PROCEDURE ConnectionHandler */

/******************************************************************/
/* Once we know what our statement is, we call out to the ap-     */
/* priate subroutine to parse and execute the SQL.                */
/******************************************************************/

PROCEDURE ParseStatement:

  DEFINE INPUT PARAMETER cSQL AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER hSocketHandle AS HANDLE NO-UNDO.

  RUN ipLog (3, "Beginning parse of " + cSQL).

  DO TRANSACTION:
         IF cSQL BEGINS "INSERT" THEN RUN insert.p (INPUT cSQL).
    ELSE IF cSQL BEGINS "DELETE" THEN RUN delete.p (INPUT cSQL).
    ELSE IF cSQL BEGINS "SELECT" THEN RUN select.p (INPUT hSocketHandle, INPUT cSQL).
    ELSE IF cSQL BEGINS "UPDATE" THEN RUN update.p (INPUT hSocketHandle, INPUT cSQL).
    ELSE IF cSQL BEGINS "HELP" THEN RUN ipHelp (INPUT hSocketHandle).
    ELSE IF cSQL BEGINS "CLOSE" THEN RUN ipDisconnect (INPUT hSocketHandle).
    ELSE IF cSQL BEGINS "STOP" THEN RUN ipStop.
  END.

END. /* PROCEDURE ParseStatement */

/******************************************************************/
/* When we get a stop command via the socket, we shutdown the     */
/* process.                                                       */
/******************************************************************/

PROCEDURE ipStop:

  IF scLogFile <> "" THEN DO:

    RUN ipLog (1, "Shutting Down by " + STRING(SELF:REMOTE-HOST) + "!").
    OUTPUT CLOSE.

  END. /* IF cLogFile <> "" */

  PAUSE 0.
  QUIT.

END. /* PROCEDURE ipStop */

/******************************************************************/
/* Help the user with syntax.                                     */
/******************************************************************/

PROCEDURE ipDisconnect:

  DEFINE INPUT PARAMETER hSocket AS HANDLE NO-UNDO.

  RUN socketWrite (hSocket, "Goodbye!" + {&CR}).
  RUN ipLog(1, "Disconnecting " + STRING(hSocket:REMOTE-HOST)).
  hSocket:DISCONNECT().

END. /* DISCONNECT */

/******************************************************************/
/* Help the user with syntax.                                     */
/******************************************************************/

PROCEDURE ipHelp:

  DEFINE INPUT PARAMETER hSocket AS HANDLE NO-UNDO.

  RUN socketWrite (hSocket, "STOP will stop the server process" + {&CR}).
  RUN socketWrite (hSocket, "CLOSE will close your connection" + {&CR}).
  RUN socketWrite (hSocket, "HELP is this message" + {&CR}).
  RUN socketWrite (hSocket, "DELETE FROM table WHERE where" + {&CR}).
  RUN socketWrite (hSocket, "Warning, DELETE WHERE needs \" instead of \' on strings" + {&CR}).

  RUN socketWrite (hSocket, "INSERT INTO table (fieldlist) VALUES (values)" + {&CR}).
  RUN socketWrite (hSocket, "It is OK to use ' on the values" + {&CR}).
  RUN socketWrite (hSocket, "Escape ' with a \\' with INSERT" + {&CR}).
  RUN socketWrite (hSocket, "UPDATE is implemented but your where clause needs to use \" for \'." + {&CR}).
  RUN socketWrite (hSocket, "SELECT can only work with one table at a time" + {&CR}).
  RUN socketWrite (hSocket, "and for the column list you can only have fields in" + {&CR}).
  RUN socketWrite (hSocket, "in that table." + {&CR}).
  RUN socketWrite (hSocket, "Your where clause needs to use \" for \'" + {&CR}).
  RUN socketWrite (hSocket, "" + {&CR}).

END. /*PROCEDURE ipHelp */
