
/******************************************************************************/
/* ProSQL - SQL integration for Progress systems                              */
/* Copyright (c) 20005, Scott Auge                                            */
/* All rights reserved.                                                       */
/*                                                                            */
/* Redistribution and use in source and binary forms, with or without         */
/* modification, are permitted provided that the following conditions         */
/* are met:                                                                   */
/*                                                                            */
/* * Redistributions of source code must retain the above copyright           */
/*   notice, this list of conditions and the following disclaimer.            */
/* * Redistributions in binary form must reproduce the above copyright        */
/*   notice, this list of conditions and the following disclaimer in          */
/*   the documentation and/or other materials provided with the distribution. */
/* * Neither the name of the Amduus Information Works, Inc. nor the names     */
/*   of its contributors may be used to endorse or promote products derived   */
/*   from this software without specific prior written permission.            */
/*                                                                            */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS        */
/* "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED  */
/* TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR */
/* PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR           */
/* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,      */
/* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,        */
/* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR         */
/* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF     */
/* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING       */
/* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS         */
/* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.               */
/******************************************************************************/

/* $headers$ 

SELECT field_list FROM table WHERE where_clause

Currently no joins are allowed!  One table at a time buddy!
Field list MUST have no spacing between commas, as in:

firstname,lastname
NOT 
firstname,lastname

That is a TODO.
*/

{logentry.i}

&GLOBAL-DEFINE CR CHR(13) + CHR(10)

DEFINE INPUT PARAMETER hSocket AS HANDLE NO-UNDO.
DEFINE INPUT PARAMETER cSQL AS CHARACTER NO-UNDO.

DEFINE VARIABLE cFieldList AS CHARACTER NO-UNDO.
DEFINE VARIABLE cTableName AS CHARACTER NO-UNDO.
DEFINE VARIABLE cWhere     AS CHARACTER NO-UNDO.
DEFINE VARIABLE cQuery     AS CHARACTER NO-UNDO.

DEFINE VARIABLE hBuffer    AS HANDLE NO-UNDO.
DEFINE VARIABLE hField     AS HANDLE NO-UNDO.
DEFINE VARIABLE hQuery     AS HANDLE NO-UNDO.

DEFINE VARIABLE iNumFields AS INTEGER NO-UNDO.
DEFINE VARIABLE iCurField  AS INTEGER NO-UNDO.
DEFINE VARIABLE cCurField  AS CHARACTER NO-UNDO.

DEFINE VARIABLE i AS INTEGER NO-UNDO.

ASSIGN cTableName = ENTRY(4, cSQL, " ")
       cFieldList = ENTRY(2, cSQL, " ")
       cWhere = TRIM(REPLACE(cSQL, "SELECT " + cFieldList + " FROM " + cTableName, "")).

ASSIGN cWhere = TRIM (cWhere)
       cWhere = REPLACE (cWhere, CHR(13), "")
       cWhere = REPLACE  (cWhere, CHR(10), "")

       cTableName = TRIM(cTableName)
       cTableName = REPLACE (cTableName, CHR(13), "")
       cTableName = REPLACE (cTableName, CHR(10), "").

RUN ipLog (2, "SELECT").
RUN ipLog (3, "Table = " + cTableName).
RUN ipLog (3, "FieldList = " + cFieldList).
RUN ipLog (3, "Where = " + cWhere).

CREATE BUFFER hBuffer FOR TABLE cTableName.
CREATE QUERY hQuery.
hQuery:ADD-BUFFER(hBuffer).

ASSIGN cQuery = "FOR EACH " + cTableName + " NO-LOCK " + cWhere.
RUN ipLog (2, "Query is: " + cQuery).

IF hQuery:QUERY-PREPARE(cQuery) = FALSE THEN DO:
    RUN ipLog(2, "Query Failed Prepare!").
    RUN SocketWrite(hSocket, "Query failed to prepare!" + {&CR}).
    RETURN.
END.

IF hQuery:QUERY-OPEN() = FALSE THEN DO:

  RUN ipLog(2, "Open FAILED!").
  /* Begin Used for debugging */
  REPEAT i = 1 TO hQuery:NUM-BUFFERS:
    hBuffer = hQuery:GET-BUFFER-HANDLE(i).
    RUN ipLog (2, "Query Has Buffer: " + hBuffer:NAME).
  END.
  RUN SocketWrite(hSocket, "Query failed to open!" + {&CR}).
  RETURN.

END. /* IF lStatusFlag = FALSE */

ASSIGN iNumFields = NUM-ENTRIES(cFieldList).
RUN SocketWrite(hSocket, "START RESULTS" + {&CR}).

REPEAT :

  hQuery:GET-NEXT.
  IF (hQuery:QUERY-OFF-END) THEN LEAVE.

  hBuffer = hQuery:GET-BUFFER-HANDLE(1).
  RUN SocketWrite(hSocket, "<row>" + {&CR}).

  DO iCurField = 1 TO iNumFields:
    ASSIGN cCurField = ENTRY(iCurField, cFieldList).
    ASSIGN hField = hBuffer:BUFFER-FIELD(cCurField).
    RUN SocketWrite(hSocket, "<column name=~"" + hField:NAME + "~">").
    RUN SocketWrite(hSocket, STRING(hField:BUFFER-VALUE)).
    RUN SocketWrite(hSocket, "</column>" + {&CR}).
  END.  /* DO iCurField */

  RUN SocketWrite(hSocket, "</row>" + {&CR}).

END. /* REPEAT */
RUN SocketWrite(hSocket, "END RESULTS" + {&CR}).

DELETE OBJECT hBuffer.
DELETE OBJECT hQuery.

/******************************************************************/
/*  Write data out to the socket!                                 */
/******************************************************************/
PROCEDURE socketWrite:

  DEFINE INPUT PARAMETER hSocket AS HANDLE NO-UNDO.
  DEFINE INPUT PARAMETER cText AS CHARACTER NO-UNDO.

  DEFINE VARIABLE mData AS MEMPTR NO-UNDO.

  SET-SIZE(mDATA) = LENGTH(cText) + 1.
  PUT-STRING(mData, 1) = cText.
  hSocket:WRITE(mData, 1, LENGTH(cText)).
  SET-SIZE (mData) = 0.

END. /* PROCEDURE socketWrite */

