
/******************************************************************************/
/* ProSQL - SQL integration for Progress systems                              */
/* Copyright (c) 20005, Scott Auge                                            */
/* All rights reserved.                                                       */
/*                                                                            */
/* Redistribution and use in source and binary forms, with or without         */
/* modification, are permitted provided that the following conditions         */
/* are met:                                                                   */
/*                                                                            */
/* * Redistributions of source code must retain the above copyright           */
/*   notice, this list of conditions and the following disclaimer.            */
/* * Redistributions in binary form must reproduce the above copyright        */
/*   notice, this list of conditions and the following disclaimer in          */
/*   the documentation and/or other materials provided with the distribution. */
/* * Neither the name of the Amduus Information Works, Inc. nor the names     */
/*   of its contributors may be used to endorse or promote products derived   */
/*   from this software without specific prior written permission.            */
/*                                                                            */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS        */
/* "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED  */
/* TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR */
/* PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR           */
/* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,      */
/* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,        */
/* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR         */
/* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF     */
/* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING       */
/* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS         */
/* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.               */
/******************************************************************************/

/* $headers$ 

Spacing is important!!!

UPDATE table SET field=value,field=value WHERE whereclause

*/

{logentry.i}

DEFINE INPUT PARAMETER hSocket AS HANDLE NO-UNDO.
DEFINE INPUT PARAMETER cSQL AS CHARACTER NO-UNDO.

DEFINE VARIABLE cTableName AS CHARACTER NO-UNDO.
DEFINE VARIABLE cWhere     AS CHARACTER NO-UNDO.
DEFINE VARIABLE cSet       AS CHARACTER NO-UNDO.
DEFINE VARIABLE cQuery     AS CHARACTER NO-UNDO.

DEFINE VARIABLE hBuffer    AS HANDLE NO-UNDO.
DEFINE VARIABLE hQuery     AS HANDLE NO-UNDO. /* Version 10 use hBuffer:FIND-FIRST */
DEFINE VARIABLE hField     AS HANDLE NO-UNDO.

DEFINE VARIABLE cTemp1     AS CHARACTER NO-UNDO.

DEFINE VARIABLE i          AS INTEGER NO-UNDO.
DEFINE VARIABLE iIter      AS INTEGER NO-UNDO.

DEFINE VARIABLE iCurField  AS INTEGER NO-UNDO.
DEFINE VARIABLE iNumFields AS INTEGER NO-UNDO.
DEFINE VARIABLE cValueList AS CHARACTER NO-UNDO.
DEFINE VARIABLE cFieldList AS CHARACTER NO-UNDO.

RUN ipLog (2, "UPDATING").

/************************************************************************/
/* Determine the table name                                             */
/************************************************************************/

ASSIGN cTableName = TRIM(ENTRY(2, cSQL, " ")).
RUN ipLog (3, "Table Name = " + cTableName).

/************************************************************************/
/* Get our WHERE Clause                                                 */
/************************************************************************/

ASSIGN cWhere = SUBSTRING (cSQL, INDEX (cSQL, " WHERE ") + 7).
RUN ipLog (3, "Where = " + cWhere).

/************************************************************************/
/* Peel out our SET tokens.                                             */
/************************************************************************/

ASSIGN cSet = cSQL
       cSet = REPLACE(cSet, "UPDATE " + cTableName + " SET ", "")
       cSet = SUBSTRING(cSet, 1, INDEX(cSet, " WHERE ")).

RUN ipLog (3, "Set = " + cSet).

RUN ipParseSet (INPUT cSet, OUTPUT cFieldList, OUTPUT cValueList).

RUN ipLog (3, "Fields = " + cFieldList).
RUN ipLog (3, "Values = " + cValueList).

/************************************************************************/
/* Start the updating process.  Remember deletes in SQL are set         */
/* oriented so we are going to look and update until no more records    */
/* matching the where clause exist.                                     */
/************************************************************************/

CREATE BUFFER hBuffer FOR TABLE cTableName.

CREATE QUERY hQuery.
hQuery:ADD-BUFFER(hBuffer).

ASSIGN cQuery = "FOR EACH " + cTableName + " EXCLUSIVE-LOCK WHERE " + cWhere.
RUN ipLog (2, "Query is: " + cQuery).

IF hQuery:QUERY-PREPARE(cQuery) = FALSE THEN DO:
    RUN ipLog(2, "Query Failed Prepare!").
END.

IF hQuery:QUERY-OPEN() = FALSE THEN DO:
  RUN ipLog(2, "Open FAILED!").
  /* Begin Used for debugging */
  REPEAT i = 1 TO hQuery:NUM-BUFFERS:
    hBuffer = hQuery:GET-BUFFER-HANDLE(i).
    RUN ipLog (2, "Query Has Buffer: " + hBuffer:NAME).
  END.

END. /* IF lStatusFlag = FALSE */

REPEAT TRANSACTION:

  hQuery:GET-NEXT.
  IF (hQuery:QUERY-OFF-END) THEN LEAVE.

  hBuffer = hQuery:GET-BUFFER-HANDLE(1).

  RUN ipLog (2, "Working RowID: " + STRING(hBuffer:ROWID)).

  ASSIGN iNumFields = NUM-ENTRIES(cFieldList, "|").
  DO iCurField = 1 TO iNumFields:

    hField = hBuffer:BUFFER-FIELD(ENTRY(iCurField, cFieldList, "|")).
 
    CASE hField:DATA-TYPE:

      WHEN "CHARACTER" THEN DO:
        ASSIGN cTemp1 = ENTRY(iCurField, cValueList, "|").
        RUN ipLog (2, "Buffer Field Name = " + hField:Name).
        RUN ipLog (2, "Integer Value = " + cTemp1).
        ASSIGN hField:BUFFER-VALUE = ENTRY(iCurField, cValueList, "|").
      END.

      WHEN "INTEGER" THEN DO:
        ASSIGN cTemp1 = ENTRY(iCurField, cValueList, "|").
        RUN ipLog (2, "Buffer Field Name = " + hField:Name).
        RUN ipLog (2, "Integer Value = " + cTemp1).
        ASSIGN hField:BUFFER-VALUE = INTEGER(ENTRY(iCurField, cValueList, "|")).
      END.

      WHEN "DECIMAL" THEN DO:
        ASSIGN cTemp1 = ENTRY(iCurField, cValueList, "|").
        RUN ipLog (2, "Buffer Field Name = " + hField:Name).
        RUN ipLog (2, "Decimal Value = " + cTemp1).
        ASSIGN hField:BUFFER-VALUE = DECIMAL(ENTRY(iCurField, cValueList, "|")).
      END.

      WHEN "LOGICAL" THEN DO:
        ASSIGN cTemp1 = ENTRY(iCurField, cValueList, "|").
        RUN ipLog (2, "Buffer Field Name = " + hField:Name).
        RUN ipLog (2, "Logical Value = " + cTemp1).
        ASSIGN hField:BUFFER-VALUE = IF cTemp1 = "true" THEN TRUE
                                     ELSE IF cTemp1 = "false" THEN FALSE
                                     ELSE ?.

      END.
    END. /* CASE */
 
  END. /* DO iCurField = 1 TO iNumFields */

END. /* REPEAT */

/******************************************************************/
/* Break apart the SET syntax into a set of field names and a set */
/* of values.                                                     */
/* A temp table would be really better for this                   */
/******************************************************************/

PROCEDURE ipParseSet:

  DEF INPUT PARAMETER cSet AS CHARACTER NO-UNDO.
  DEF OUTPUT PARAMETER cFieldList AS CHARACTER NO-UNDO.
  DEF OUTPUT PARAMETER cValueList AS CHARACTER NO-UNDO.

  DEFINE VARIABLE lIsString    AS LOGICAL NO-UNDO.
  DEFINE VARIABLE lIsFieldName AS LOGICAL NO-UNDO.
  DEFINE VARIABLE iIter        AS INTEGER NO-UNDO.
  DEFINE VARIABLE iMaxIter     AS INTEGER NO-UNDO.
  DEFINE VARIABLE cCurChar     AS CHARACTER NO-UNDO.
  DEFINE VARIABLE cLastChar    AS CHARACTER NO-UNDO.

  ASSIGN iMaxIter = LENGTH(cSet)
         lIsString = NO
         lIsFieldName = YES
         cLastChar = "".

  DO iIter = 1 TO iMaxIter:

    ASSIGN cCurChar = SUBSTRING(cSet, iIter, 1).

    IF lIsFieldName AND cCurChar = " " THEN NEXT.

    IF lIsString AND cCurChar = "'" AND cLastChar <> "\\" THEN DO:
      ASSIGN lIsString = NOT lIsString.
      NEXT.
    END.

    IF NOT lIsString AND cCurChar = "," THEN DO:
      lIsFieldName = TRUE.
      ASSIGN cValueList = cValueList + "|".
      NEXT.
    END.

    IF lIsFieldName AND cCurChar = "=" THEN DO:
      lIsFieldName = NOT lIsFieldName.
      cFieldList = cFieldList + "|".
      NEXT.
    END.

    IF lIsFieldName THEN ASSIGN cFieldList = cFieldList + cCurChar.

    IF NOT lIsFieldName THEN ASSIGN cValueList = cValueList + cCurChar.

    ASSIGN cLastChar = cCurChar.

  END. /* DO iIter = 1 */

  ASSIGN cFieldList = SUBSTRING(cFieldList, 1, LENGTH(cFieldList) - 1).

  RUN ipCleanValues (INPUT cValueList, OUTPUT cValueList).

END. /* PROCEDURE ipParseSet */

/***********************************************************************/
/* The stuff coming out of ipParseSet is nice, but still has the ' on  */
/* between the | delimiters.  So if we need them there, it is cool, if */
/* note - we need to take that and the spaces out of the picture.      */
/***********************************************************************/

PROCEDURE ipCleanValues:
  
  DEF INPUT PARAMETER cListIn   AS CHARACTER NO-UNDO.
  DEF OUTPUT PARAMETER cListOut AS CHARACTER NO-UNDO.

  DEFINE VARIABLE iCurChar AS INTEGER NO-UNDO.
  DEFINE VARIABLE cCurChar AS CHARACTER NO-UNDO.
  DEFINE VARIABLE iNumChar AS INTEGER NO-UNDO.

  DEFINE VARIABLE cLastChar AS CHARACTER NO-UNDO.

  DEFINE VARIABLE lIsString AS LOGICAL NO-UNDO.

  ASSIGN iNumChar = LENGTH(cListIn)
         lIsString = NO.

  DO iCurChar = 1 TO iNumChar:

    ASSIGN cCurChar = SUBSTRING(cListIn, iCurChar, 1).

    IF NOT lIsString AND cCurChar = " " THEN NEXT.

    IF cCurChar = "\\" THEN DO:
      ASSIGN cLastChar = "\\".
      NEXT.
    END.

    IF cCurChar = "'" AND cLastChar <> "\\" THEN DO:
      ASSIGN lIsString = NOT lIsString.
      NEXT.
    END.

    ASSIGN cLastChar = cCurChar
           cListOut = cListOut + cCurChar.

  END. /* DO iCurChar = 1 TO ... */

END. /* PROCEDURE ipCleanValues */

